@@ -563,6 +563,8 @@ static const char *Oops_arch_to_eip(const OPTIONS *options)
     if (strstr(options->target, "i370") ||
         strstr(options->target, "s390"))
 	return("PSW");
+    if (strstr(options->target, "ia64"))
+	return("IP");
     return("???");
 }
 
@@ -581,6 +583,8 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
     static regmatch_t *re_Oops_eip_mips_pmatch;
     static regex_t     re_Oops_eip_kdb;
     static regmatch_t *re_Oops_eip_kdb_pmatch;
+    static regex_t     re_Oops_eip_ia64;
+    static regmatch_t *re_Oops_eip_ia64_pmatch;
     static regex_t     re_Oops_eip_other;
     static regmatch_t *re_Oops_eip_other_pmatch;
     static const char procname[] = "Oops_eip";
@@ -671,14 +675,42 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
 	return((*string)[re_Oops_eip_mips.re_nsub]);
     }
 
+    /* Oops 'IP' line for ia64, space, ip, optional white space, ':',
+     * optional white space, bracketed address.
+     */
+    RE_COMPILE(&re_Oops_eip_ia64,
+	    " ip *:+ *"
+	    BRACKETED_ADDRESS,
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_eip_ia64_pmatch);
+
+    i = regexec(&re_Oops_eip_ia64, line, re_Oops_eip_ia64.re_nsub+1,
+	re_Oops_eip_ia64_pmatch, 0);
+    DEBUG(4, "regexec ia64 %d", i);
+    if (i == 0) {
+	re_string_check(re_Oops_eip_ia64.re_nsub+1, string_max,
+	    procname);
+	re_strings(&re_Oops_eip_ia64, line, re_Oops_eip_ia64_pmatch,
+	    string);
+	return((*string)[re_Oops_eip_ia64.re_nsub]);
+    }
+
     /* SGI kdb backtrace EIP.  At start of line, 0x unbracketed
      * address, 0x unbracketed address, anything, '+0x'.  The second
      * unbracketed address is the EIP.
+     * The initial kdb reports has lines like
+     * Entering kdb on processor 2 due to panic @ 0x5005f426
      */
     RE_COMPILE(&re_Oops_eip_kdb,
-	    "^0x" UNBRACKETED_ADDRESS	/* 1 */
-	    "0x" UNBRACKETED_ADDRESS	/* 2 */
-	    ".*+0x",
+	    "("					/* 1 */
+	      "^0x" UNBRACKETED_ADDRESS		/* 2 */
+	      "0x" UNBRACKETED_ADDRESS		/* 3 */
+	      ".*+0x"
+	    ")"
+	    "|("				/* 4 */
+	      "^Entering kdb on processor.*0x"
+	      UNBRACKETED_ADDRESS		/* 5 */
+	    ")",
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_eip_kdb_pmatch);
 
@@ -690,7 +722,10 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
 	    procname);
 	re_strings(&re_Oops_eip_kdb, line, re_Oops_eip_kdb_pmatch,
 	    string);
-	return((*string)[2]);
+	if ((*string)[3] && *((*string)[3]))
+	    return((*string)[3]);
+	if ((*string)[5] && *((*string)[5]))
+	    return((*string)[5]);
     }
 
     /* Oops 'EIP:' line for other architectures */
@@ -825,6 +860,48 @@ static void Oops_set_mips_ra(const char *value, SYMBOL_SET *ss)
     add_symbol_n(ss, ra, 'R', 1, ">>RA; ");
 }
 
+/* Look for the ia64 b0 line and set all the values */
+static void Oops_set_ia64_b0(const char *line, char ***string, int string_max, SYMBOL_SET *ss)
+{
+    int i;
+    addr_t b;
+    char label[6];
+    static regex_t     re_Oops_ia64_b0;
+    static regmatch_t *re_Oops_ia64_b0_pmatch;
+    static const char procname[] = "Oops_ia64_b0";
+
+    /* Oops 'b0' line for ia64, b0 : unbracketed address, may be
+     * repeated for b6, b7. 
+     */
+    RE_COMPILE(&re_Oops_ia64_b0,
+	    "^b[0-7] *: *"
+	    UNBRACKETED_ADDRESS,
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_ia64_b0_pmatch);
+
+    /* Scan entire line for b0, b6, b7 */
+    while(1) {
+	i = regexec(&re_Oops_ia64_b0, line, re_Oops_ia64_b0.re_nsub+1,
+	    re_Oops_ia64_b0_pmatch, 0);
+	DEBUG(4, "regexec %d", i);
+	if (i)
+	    break;
+	re_string_check(re_Oops_ia64_b0.re_nsub+1, string_max, procname);
+	re_strings(&re_Oops_ia64_b0, line, re_Oops_ia64_b0_pmatch, string);
+	strcpy(label, ">>bx; ");
+	label[3] = line[1];	/* register number */
+	b = hexstring((*string)[re_Oops_ia64_b0.re_nsub]);
+	if (errno) {
+	    ERROR(" Invalid hex value in b%c line, ignored - '%s'",
+		line[1], (*string)[re_Oops_ia64_b0.re_nsub]);
+	    perror(prefix);
+	    b = 0;
+	}
+	add_symbol_n(ss, b, 'R', 1, label);
+	line += re_Oops_ia64_b0_pmatch[0].rm_eo;
+    }
+}
+
 /* Look for the sparc o7/i7 registers line, returns start of the relevant hex
  * value.
  */
@@ -929,10 +1006,13 @@ static char *Oops_i370_psw(const char *line, char ***string, int string_max)
 
 /* Set the i370 PSW from the PSW line */
 static void Oops_set_i370_psw(const char *value, SYMBOL_SET *ss,
-                              const OPTIONS *options)
+                              const char *me, const bfd *ibfd,
+                              OPTIONS *options)
 {
     static const char procname[] = "Oops_set_i370_psw";
     addr_t PSW;
+    /* Ensure target is set so EIP text is correct */
+    Oops_set_default_ta(me, ibfd, options);
     PSW = hexstring(value);
     if (errno) {
 	ERROR(" Invalid hex value in PSW line, ignored - '%s'", value);
@@ -951,11 +1031,11 @@ static char *Oops_s390_psw(const char *line, char ***string, int string_max)
     static regmatch_t *re_Oops_s390_psw_pmatch;
     static const char procname[] = "Oops_s390_psw";
 
-    /* Oops 'User PSW:' line for s390, PSW at start of line, value is
+    /* Oops 'Kernel PSW:' line for s390, PSW at start of line, value is
      * second address.
      */
     RE_COMPILE(&re_Oops_s390_psw,
-	    "^User PSW: *"
+	    "^Kernel PSW: *"
 	    UNBRACKETED_ADDRESS		/* flags */
 	    UNBRACKETED_ADDRESS,
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
@@ -974,8 +1054,8 @@ static char *Oops_s390_psw(const char *line, char ***string, int string_max)
 }
 
 /* No need for Oops_set_s390_psw, can use Oops_set_i370_psw for both */
-#define Oops_set_s390_psw(line, string, string_max) \
-        Oops_set_i370_psw(line, string, string_max)
+#define Oops_set_s390_psw(line, string, me, ibfd, options) \
+        Oops_set_i370_psw(line, string, me, ibfd, options)
 
 /* Look for the Trace multilines :(.  Returns start of addresses. */
 static const char *Oops_trace(const char *line)
@@ -1203,7 +1283,7 @@ static const char *Oops_s390_regs(const char *line)
     static const char procname[] = "Oops_s390_regs";
 
     RE_COMPILE(&re_Oops_s390_regs,
-	    "^(User GPRS.*)",
+	    "^(Kernel GPRS.*)",
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_s390_regs_pmatch);
 
@@ -1447,7 +1527,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* i386 2.0 */  "|Corrupted stack page"
     /* i386 */	    "|invalid operand: "
     /* i386 */	    "|Oops: "
-    /* i386 */	    "|Cpu: +[0-9]"
+    /* i386 */	    "|Cpu:* +[0-9]"
     /* i386 */	    "|current->tss"
     /* i386 */	    "|\\*pde +="
     /* i386 */	    "|EIP: "
@@ -1458,6 +1538,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* i386 */	    "|CR0: "
     /* i386 */	    "|wait_on_"
     /* i386 */	    "|irq: "
+    /* i386 */	    "|Stack dumps:"
 
     /* m68k */	    "|pc[:=]"
     /* m68k */	    "|68060 access"
@@ -1533,13 +1614,32 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* i370 */	    "|Kernel stack "
     /* any other i370 lines to print? */
 
+    /* s390 */	    "|User process fault:"
+    /* s390 */	    "|failing address"
     /* s390 */	    "|User PSW"
     /* s390 */	    "|User GPRS"
     /* s390 */	    "|User ACRS"
+    /* s390 */	    "|Kernel PSW"
+    /* s390 */	    "|Kernel GPRS"
+    /* s390 */	    "|Kernel ACRS"
     /* s390 */	    "|illegal operation"
     /* s390 */	    "|task:"
     /* any other s390 lines to print? */
 
+    /* SGI kdb */ "|Entering kdb"
+    /* SGI kdb */ "|eax *="
+    /* SGI kdb */ "|esi *="
+    /* SGI kdb */ "|ebp *="
+    /* SGI kdb */ "|ds *="
+    /* any other SGI kdb lines to print? */
+
+    /* ia64 */	    "|psr *:"
+    /* ia64 */	    "|unat *: "
+    /* ia64 */	    "|rnat *: "
+    /* ia64 */	    "|ldrs *: "
+    /* ia64 */	    "|[bfr][0-9]+ *: "
+    /* any other mips lines to print? */
+
 		")",
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_print_s_pmatch);
@@ -1629,6 +1729,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* various */   "|Aiee"      /* anywhere in text is a bad sign (TM) */
     /* various */   "|die_if_kernel"  /* ditto */
     /* various */   "|NMI "
+    /* various */   "|BUG "
 
     /* alpha */     "|\\([0-9]\\): Oops "
     /* alpha */     "|: memory violation"
@@ -1925,15 +2026,16 @@ int Oops_read(OPTIONS *options)
 		if ((start = Oops_sparc_oi7(text, &string, MAX_STRINGS)))
 		    Oops_set_sparc_oi7(start, &string, &ss_format);
 		if ((start = Oops_i370_psw(text, &string, MAX_STRINGS)))
-		    Oops_set_i370_psw(start, &ss_format, options);
+		    Oops_set_i370_psw(start, &ss_format, me, ibfd, options);
 		if ((start = Oops_s390_psw(text, &string, MAX_STRINGS)))
-		    Oops_set_s390_psw(start, &ss_format, options);
+		    Oops_set_s390_psw(start, &ss_format, me, ibfd, options);
 		if ((start = Oops_trace(text)))
 		    Oops_trace_line(text, start, &ss_format);
 		if ((start = Oops_i370_regs(text)))
 		    Oops_set_i370_regs(text, start, &ss_format, options);
 		if ((start = Oops_s390_regs(text)))
 		    Oops_set_s390_regs(text, start, &ss_format, options);
+		Oops_set_ia64_b0(text, &string, MAX_STRINGS, &ss_format);
 		if ((start = Oops_code(text, &string, MAX_STRINGS))) {
 		    /* In case no EIP line was seen */
 		    Oops_set_default_ta(me, ibfd, options);
